/*
 * DigitGUI.java
 *
 * Created on May 24, 2007, 10:18 PM
 * 
 * Author: Tomasz Gebarowski
 */

package gui;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;
import kb.KnowledgeBase;
import nn.BackPropagationNN;

/**
 * Digit Recognizer Neural Network Self Learning Tool
 * Graphical User Interface Class
 * @author  Tomasz Gebarowski
 */
public class DigitGUI extends javax.swing.JFrame implements ChangeListener {
    
    // Default number of steps for each training
    private int totalSteps = 10000;
    
    // Reference to Neural Network System using Back Propagation method as learning technic
    private BackPropagationNN nn;
    
    // Reference to KnolwedgeBase resposnible for reading digits data from external file
    private KnowledgeBase kb;
    
    // Determines if application is in reset state ( no data loaded to neural network structure )
    private boolean reset;
    
    /** Creates new form DigitGUI */
    public DigitGUI() {
        initComponents();
        
        // Set Progress Bar properties
        progressBar.setMinimum(0);
        progressBar.setMaximum(100);
        progressBar.setStringPainted(true);
        
        // Set Reset state and disable given buttons
        updateButtons(true);
        
        
        //Set new rendering style for JTable components ( black dots )
        for ( int icol = 0; icol < 5; icol++ ) {
            TableColumn col = this.testDataTable.getColumnModel().getColumn(icol);
            col.setCellRenderer(new CellRenderer() );
            
            col = this.caseDataTable.getColumnModel().getColumn(icol);
            col.setCellRenderer(new CellRenderer() );
        }
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        tabbedPane = new javax.swing.JTabbedPane();
        trainingPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        startTrainingButton = new javax.swing.JButton();
        loadButton = new javax.swing.JButton();
        numberOfSymbolsLabel = new javax.swing.JLabel();
        numberOfIterationsLabel = new javax.swing.JLabel();
        numberOfHiddenNodesLabel = new javax.swing.JLabel();
        loadHexButton = new javax.swing.JButton();
        loadBinButton = new javax.swing.JButton();
        testPanel = new javax.swing.JPanel();
        testDataPane = new javax.swing.JScrollPane();
        testDataTable = new javax.swing.JTable();
        jLabel4 = new javax.swing.JLabel();
        recognizedDigit = new javax.swing.JLabel();
        recognizeButton = new javax.swing.JButton();
        recognizeStatusLabel = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        clearTestField = new javax.swing.JButton();
        selfTeacherPanel = new javax.swing.JPanel();
        caseDataTable = new javax.swing.JTable();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        suggestedOutput = new javax.swing.JComboBox();
        addTestCaseButton = new javax.swing.JButton();
        resetButton = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        numberOfCasesLabel = new javax.swing.JLabel();
        clearCaseField = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        titleLabel = new javax.swing.JLabel();
        authorsTitleLabel = new javax.swing.JLabel();
        author1Label = new javax.swing.JLabel();
        appVersionLabel = new javax.swing.JLabel();
        licenceLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jLabel1.setText("Number of training symbols:");

        jLabel2.setText("Number of iterations:");

        jLabel3.setText("Number of hidden layers:");

        startTrainingButton.setText("Start");
        startTrainingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startTrainingButtonActionPerformed(evt);
            }
        });

        loadButton.setText("Load Digits [0-9]");
        loadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadButtonActionPerformed(evt);
            }
        });

        numberOfSymbolsLabel.setText("0");

        numberOfIterationsLabel.setText("0");

        numberOfHiddenNodesLabel.setText("0");

        loadHexButton.setText("Load Hex Digits [0-F]");
        loadHexButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadHexButtonActionPerformed(evt);
            }
        });

        loadBinButton.setText("Load Binary Digits [0-1]");
        loadBinButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadBinButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout trainingPanelLayout = new org.jdesktop.layout.GroupLayout(trainingPanel);
        trainingPanel.setLayout(trainingPanelLayout);
        trainingPanelLayout.setHorizontalGroup(
            trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(trainingPanelLayout.createSequentialGroup()
                .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, trainingPanelLayout.createSequentialGroup()
                        .add(20, 20, 20)
                        .add(progressBar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 575, Short.MAX_VALUE))
                    .add(trainingPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(startTrainingButton))
                    .add(trainingPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel1)
                            .add(jLabel2)
                            .add(jLabel3))
                        .add(23, 23, 23)
                        .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(numberOfSymbolsLabel)
                            .add(numberOfHiddenNodesLabel)
                            .add(numberOfIterationsLabel)))
                    .add(trainingPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(loadButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 186, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(loadHexButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 202, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(loadBinButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 178, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        trainingPanelLayout.setVerticalGroup(
            trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(trainingPanelLayout.createSequentialGroup()
                .add(17, 17, 17)
                .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(loadButton)
                    .add(loadHexButton)
                    .add(loadBinButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(numberOfSymbolsLabel))
                .add(13, 13, 13)
                .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(numberOfIterationsLabel))
                .add(16, 16, 16)
                .add(trainingPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(numberOfHiddenNodesLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 22, Short.MAX_VALUE)
                .add(startTrainingButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        tabbedPane.addTab("Trainer", trainingPanel);

        testDataTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)}
            },
            new String [] {
                "", "", "", "", ""
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        testDataPane.setViewportView(testDataTable);

        jLabel4.setText("Recognized Digit:");

        recognizedDigit.setFont(new java.awt.Font("Arial Narrow", 0, 48));
        recognizedDigit.setText("?");

        recognizeButton.setText("Recognize");
        recognizeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                recognizeButtonActionPerformed(evt);
            }
        });

        recognizeStatusLabel.setForeground(new java.awt.Color(204, 0, 0));
        recognizeStatusLabel.setText("Network should be trained before performing recognition");

        jLabel8.setText("Input Digit:");

        clearTestField.setText("Clear");
        clearTestField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearTestFieldActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout testPanelLayout = new org.jdesktop.layout.GroupLayout(testPanel);
        testPanel.setLayout(testPanelLayout);
        testPanelLayout.setHorizontalGroup(
            testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(testPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(testPanelLayout.createSequentialGroup()
                        .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel8)
                            .add(testPanelLayout.createSequentialGroup()
                                .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, clearTestField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .add(org.jdesktop.layout.GroupLayout.LEADING, testDataPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 87, Short.MAX_VALUE))
                                .add(55, 55, 55)
                                .add(recognizeButton)))
                        .add(61, 61, 61)
                        .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(recognizedDigit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 74, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel4)))
                    .add(recognizeStatusLabel))
                .addContainerGap(175, Short.MAX_VALUE))
        );
        testPanelLayout.setVerticalGroup(
            testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(testPanelLayout.createSequentialGroup()
                .add(20, 20, 20)
                .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel4)
                    .add(jLabel8))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(testPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(recognizeButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 49, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(recognizedDigit, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 59, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(testDataPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 119, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(clearTestField)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 13, Short.MAX_VALUE)
                .add(recognizeStatusLabel))
        );

        tabbedPane.addTab("Test ( Drawing )", testPanel);

        caseDataTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)},
                {new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false), new Boolean(false)}
            },
            new String [] {
                "", "", "", "", ""
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });

        jLabel5.setText("New Test Case:");

        jLabel6.setText("Suggested output:");

        suggestedOutput.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15" }));

        addTestCaseButton.setText("Add Case");
        addTestCaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                addTestCaseButtonActionPerformed(evt);
            }
        });

        resetButton.setText("Reset");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        jLabel7.setText("Number of test cases:");

        numberOfCasesLabel.setText("0");

        clearCaseField.setText("Clear");
        clearCaseField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearCaseFieldActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout selfTeacherPanelLayout = new org.jdesktop.layout.GroupLayout(selfTeacherPanel);
        selfTeacherPanel.setLayout(selfTeacherPanelLayout);
        selfTeacherPanelLayout.setHorizontalGroup(
            selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(selfTeacherPanelLayout.createSequentialGroup()
                .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, selfTeacherPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .add(clearCaseField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, selfTeacherPanelLayout.createSequentialGroup()
                        .add(22, 22, 22)
                        .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel5)
                            .add(caseDataTable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 96, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .add(68, 68, 68)
                .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel6)
                    .add(selfTeacherPanelLayout.createSequentialGroup()
                        .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(selfTeacherPanelLayout.createSequentialGroup()
                                .add(jLabel7)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(numberOfCasesLabel))
                            .add(suggestedOutput, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 163, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(28, 28, 28)
                        .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(resetButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE)
                            .add(addTestCaseButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE))))
                .addContainerGap(118, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        selfTeacherPanelLayout.setVerticalGroup(
            selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(selfTeacherPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(jLabel6))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(selfTeacherPanelLayout.createSequentialGroup()
                        .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(suggestedOutput, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(addTestCaseButton))
                        .add(17, 17, 17)
                        .add(selfTeacherPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel7)
                            .add(numberOfCasesLabel)
                            .add(resetButton)))
                    .add(caseDataTable, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(clearCaseField)
                .add(38, 38, 38))
        );

        tabbedPane.addTab("Add New Case", selfTeacherPanel);

        titleLabel.setFont(new java.awt.Font("Lucida Grande", 0, 18));
        titleLabel.setText("Digit Recognizing Self Learning Tool");

        authorsTitleLabel.setText("Authors:");

        author1Label.setText("Tomasz Gebarowski");

        appVersionLabel.setText("v. 1.0.");

        licenceLabel.setText("This software can be freely distributed without any restrictions");

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(12, 12, 12)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(authorsTitleLabel)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(titleLabel)
                                .add(17, 17, 17)
                                .add(appVersionLabel))))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(65, 65, 65)
                        .add(author1Label))
                    .add(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(licenceLabel)))
                .addContainerGap(197, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(titleLabel)
                    .add(appVersionLabel))
                .add(21, 21, 21)
                .add(authorsTitleLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(author1Label)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 100, Short.MAX_VALUE)
                .add(licenceLabel)
                .add(9, 9, 9))
        );

        tabbedPane.addTab("About", jPanel1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(tabbedPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 634, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(tabbedPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 274, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /****** EVENTS HANDLING ROUTINES *******************************************/
    
    /** Clear Case JTable Component, remove all selected pixels */
    private void clearCaseFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearCaseFieldActionPerformed
        TableModel tm = this.caseDataTable.getModel();
        
        for( int  col = 0; col < caseDataTable.getColumnCount(); col++)
            for ( int row = 0; row < caseDataTable.getRowCount(); row++ )
                tm.setValueAt(new Boolean(false), row, col);
    }//GEN-LAST:event_clearCaseFieldActionPerformed

    /** Clear Test JTable Component, remove all selected pixels */
    private void clearTestFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearTestFieldActionPerformed
        TableModel tm = this.testDataTable.getModel();
        
        for( int  col = 0; col < testDataTable.getColumnCount(); col++)
            for ( int row = 0; row < testDataTable.getRowCount(); row++ )
                tm.setValueAt(new Boolean(false), row, col);
        
    }//GEN-LAST:event_clearTestFieldActionPerformed

    /** Load files responsible for teaching binary digits */
    private void loadBinButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadBinButtonActionPerformed
        resetButtonActionPerformed(evt);
        this.load("bin");
    }//GEN-LAST:event_loadBinButtonActionPerformed

    /** Load files responsible from teaching hexadecimal digits */
    private void loadHexButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadHexButtonActionPerformed
        resetButtonActionPerformed(evt);
        this.load("hex");
    }//GEN-LAST:event_loadHexButtonActionPerformed

    /** Load files responsible for teaching decimal digits */
    private void loadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadButtonActionPerformed
        resetButtonActionPerformed(evt);
        this.load("dec");
    }//GEN-LAST:event_loadButtonActionPerformed

   
    /** Execute while Reset button is pressed */
    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed

        numberOfCasesLabel.setText("0"); 
        numberOfHiddenNodesLabel.setText("0");
        numberOfSymbolsLabel.setText("0");
        updateButtons(true);     
       
        clearCaseField.doClick();
        clearTestField.doClick();
    }//GEN-LAST:event_resetButtonActionPerformed

    /** Handling "Add New Case" button */
    private void addTestCaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_addTestCaseButtonActionPerformed

        // If the neural network is not constructed then create it
        if ( reset ) {
            kb = new KnowledgeBase();
            kb.load("dec");
            
            nn = new BackPropagationNN(kb.getNumberOfInputs(), kb.getNumberOfInputs(), kb.getNumberOfOutputs());  
        }
        
        // Create input array
        double[] inputs = new double[kb.getNumberOfInputs()];

        int pos = 0;       
        
        //Add new case from caseDataTable (JTable)
        for ( int irow = 0; irow < caseDataTable.getRowCount(); irow++ ) {
            for ( int icol = 0; icol < caseDataTable.getColumnCount(); icol++ ) {              
                Boolean value = (Boolean)this.caseDataTable.getModel().getValueAt(irow,icol);
                boolean val = value.booleanValue();
                inputs[pos] = getDouble( val);
                pos++;
            }
        }
        
        // Creating space for Output
        double[] outputs = kb.binaryDigit( (String)suggestedOutput.getSelectedItem() );   
        
        
        // Neural network configuration
        nn.setSteps(totalSteps);
        nn.addCase(inputs, outputs);
        
        // Adding change listener - handling progress bar
        nn.addChangeListener(this);

        // Update Buttons & Labels
        updateButtons(false);
        updateLabels();
        this.recognizeButtonActivation(false);
            
    }//GEN-LAST:event_addTestCaseButtonActionPerformed

    /** Handling "Recognize" button press event */
    private void recognizeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_recognizeButtonActionPerformed
        
        /** Create space for inputs */
        double[] inputs = new double[kb.getNumberOfInputs()];

        int pos = 0;
       
        
        /** Get data from testDataTable and store it in inputs array */
        for ( int irow = 0; irow < testDataTable.getRowCount(); irow++ ) {
            for ( int icol = 0; icol < testDataTable.getColumnCount(); icol++ ) {              
                Boolean value = (Boolean)this.testDataTable.getModel().getValueAt(irow,icol);
                boolean val = value.booleanValue();
                inputs[pos] = getDouble( val);
                pos++;
            }
        }
        
        /** Peform forward push on neural network */
        nn.processForward(inputs);

        /** Output recognized digit */
        this.recognizedDigit.setText( String.valueOf(  getDigit( nn.outputs ) ) );
        
    }//GEN-LAST:event_recognizeButtonActionPerformed

    /** Start Button support */
    private void startTrainingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startTrainingButtonActionPerformed
       
            // Create new thread for teaching process
            Thread t = new  Thread(nn);
            t.start(); // run thread

            
            //Update Labels & Buttons
            updateLabels();  
            this.recognizeButtonActivation(true);
    }//GEN-LAST:event_startTrainingButtonActionPerformed
    
    
    
     /**
     * Routine responsible for loading data to neural knowledge base
     * cat  String Directory in which configuration files are stored
     */ 
    private void load( String cat  ) {
        if ( reset ) {
                      
            kb = new KnowledgeBase();
            kb.load(cat);

            nn = new BackPropagationNN(kb.getNumberOfInputs(), kb.getNumberOfInputs(), kb.getNumberOfOutputs());
            nn.addChangeListener(this);
            nn.addTest(totalSteps, kb.getKB() );
       
            updateButtons(false);
            recognizeButtonActivation(false);
            updateLabels();
        }          
    }
    
    /** Update labels */
    private void updateLabels() {
            this.numberOfSymbolsLabel.setText( String.valueOf(nn.getNumberOfTestCases())  );           
            this.numberOfHiddenNodesLabel.setText(String.valueOf( nn.getNumberOfHiddens()));
            this.numberOfCasesLabel.setText( String.valueOf( nn.getNumberOfTestCases() ) );
    }
    
    /** Update buttons state basing on reset flag
     *  value boolea New value of reset flag
     */
    private void updateButtons(boolean value) {
        this.reset = value;
        recognizeButton.setEnabled(!reset);
        startTrainingButton.setEnabled(!reset);
        resetButton.setEnabled(!reset);
    }
    
    /** Routine responsbile for managing state of "Recognize" button
     *  state boolean State deciding whether button is enabled or disabled
     */
    private void recognizeButtonActivation( boolean state ) {
        recognizeButton.setEnabled( state );
        
        if ( state ) {
            this.recognizeStatusLabel.setText("");
        } else {
            this.recognizeStatusLabel.setText("Network should be trained before performing recognition");
        }
    }
    
    /* Handle State Changed Event used for changing progress bar */
    public void stateChanged( ChangeEvent e ) {
        
        // Calculate progress
        int progress = (int) ( ( ( 1.0 * ( nn.currentStep + 2 ) / totalSteps ) ) * 100.0 ) ;
        
        // Set current value on progress bar
        progressBar.setValue( progress );
        progressBar.setString( String.valueOf( progress ) + "%" );
        
        this.numberOfIterationsLabel.setText( String.valueOf(  nn.currentStep + 2) );
    }     
    
    /** Convert boolean to double
     *  @param value boolean boolean value
     *  @return double Double representation of boolean value */
    public double getDouble(boolean value ) {
        if ( value )
            return 1.0;
        
        return 0.0;
    }
    
    
    /** Get decimal digit from binary representation
     *  @param outputs[] double Binary representation of a number stored in array
     *  @return int Resulting decimal
     */
    public int getDigit( double [] outputs ) {
        
        int value = 0;
        
        // Bin to Int conversion
        for ( int i = 0; i < 4; i++ ) {
            value += Math.round(outputs[i]) * Math.pow(2, i);
        }
        
        return value;
    }
  
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton addTestCaseButton;
    private javax.swing.JLabel appVersionLabel;
    private javax.swing.JLabel author1Label;
    private javax.swing.JLabel authorsTitleLabel;
    private javax.swing.JTable caseDataTable;
    private javax.swing.JButton clearCaseField;
    private javax.swing.JButton clearTestField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel licenceLabel;
    private javax.swing.JButton loadBinButton;
    private javax.swing.JButton loadButton;
    private javax.swing.JButton loadHexButton;
    private javax.swing.JLabel numberOfCasesLabel;
    private javax.swing.JLabel numberOfHiddenNodesLabel;
    private javax.swing.JLabel numberOfIterationsLabel;
    private javax.swing.JLabel numberOfSymbolsLabel;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JButton recognizeButton;
    private javax.swing.JLabel recognizeStatusLabel;
    private javax.swing.JLabel recognizedDigit;
    private javax.swing.JButton resetButton;
    private javax.swing.JPanel selfTeacherPanel;
    private javax.swing.JButton startTrainingButton;
    private javax.swing.JComboBox suggestedOutput;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JScrollPane testDataPane;
    private javax.swing.JTable testDataTable;
    private javax.swing.JPanel testPanel;
    private javax.swing.JLabel titleLabel;
    private javax.swing.JPanel trainingPanel;
    // End of variables declaration//GEN-END:variables
    
}
